@@ -4,7 +4,7 @@
  * Gnome Chemisty Utils
  * crystaldoc.cc 
  *
- * Copyright (C) 2002-2006 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2002-2007 Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -22,16 +22,17 @@
  * USA
  */
 
+#include "config.h"
 #include "crystaldoc.h"
 #include "crystalview.h"
 #include "xml-utils.h"
-#include <locale.h>
-#include <string.h>
-#include <vector>
-#include <math.h>
 #include <gtk/gtk.h>
 #include <libintl.h>
-#define _(String) gettext (String)
+#include <glib/gi18n-lib.h>
+#include <cstring>
+#include <vector>
+#include <clocale>
+#include <cmath>
 
 #define __max(x,y)  ((x) > (y)) ? (x) : (y)
 #define __min(x,y)  ((x) < (y)) ? (x) : (y)
@@ -39,20 +40,22 @@
 
 using namespace gcu;
 
-gchar *LatticeName[] = {"simple cubic",
-	"body-centered cubic",
-	"face-centered cubic",
-	"hexagonal",
-	"tetragonal",
-	"body-centered tetragonal",
-	"orthorhombic",
-	"base-centered orthorhombic",
-	"body-centered orthorhombic",
-	"face-centered orthorhombic",
-	"rhombohedral",
-	"monoclinic",
-	"base-centered monoclinic",
-	"triclinic"};
+gchar *gcu::LatticeName[] = {
+	(gchar*) "simple cubic",
+	(gchar*) "body-centered cubic",
+	(gchar*) "face-centered cubic",
+	(gchar*) "hexagonal",
+	(gchar*) "tetragonal",
+	(gchar*) "body-centered tetragonal",
+	(gchar*) "orthorhombic",
+	(gchar*) "base-centered orthorhombic",
+	(gchar*) "body-centered orthorhombic",
+	(gchar*) "face-centered orthorhombic",
+	(gchar*) "rhombohedral",
+	(gchar*) "monoclinic",
+	(gchar*) "base-centered monoclinic",
+	(gchar*) "triclinic"
+};
 
 CrystalDoc::CrystalDoc (Application *App): GLDocument (App)
 {
@@ -231,6 +234,7 @@ void CrystalDoc::ParseXMLTree (xmlNode* xml)
 	}
 	setlocale (LC_NUMERIC, old_num_locale);
 	g_free (old_num_locale);
+	SetDirty (false);
 	Update ();
 }
 
@@ -251,7 +255,7 @@ CrystalView *CrystalDoc::GetView()
 
 void CrystalDoc::Update()
 {
-	m_bEmpty = (AtomDef.empty() && LineDef.empty()) ? true : false;
+	m_Empty = (AtomDef.empty() && LineDef.empty()) ? true : false;
 	CrystalAtom Atom;
 	CrystalLine Line;
 	gdouble alpha = m_alpha * M_PI / 180;
@@ -272,14 +276,11 @@ void CrystalDoc::Update()
 
 	////////////////////////////////////////////////////////////
 	//Establish list of atoms
-	CrystalAtomList::iterator i;
+	CrystalAtomList::iterator i, iend = AtomDef.end ();
 	
-	for (i = AtomDef.begin(); i != AtomDef.end(); i++)
-	{
-		
+	for (i = AtomDef.begin(); i != iend; i++) {
 		Duplicate(**i);
-		switch (m_lattice)
-		{
+		switch (m_lattice) {
 		case body_centered_cubic:
 		case body_centered_tetragonal:
 		case body_centered_orthorhombic:
@@ -308,8 +309,8 @@ void CrystalDoc::Update()
 	
 	////////////////////////////////////////////////////////////
 	//Establish list of atoms
-	CrystalLineList::iterator j;
-	for (j = LineDef.begin() ; j != LineDef.end() ; j++)
+	CrystalLineList::iterator j, jend = LineDef.end();
+	for (j = LineDef.begin() ; j != jend ; j++)
 	{
 		switch ((*j)->Type())
 		{
@@ -379,10 +380,6 @@ void CrystalDoc::Update()
 		}
 	}
 
-	//Searching the center of the crystal
-	Atom.SetCoords((m_xmax +m_xmin) / 2, (m_ymax + m_ymin) / 2, (m_zmax + m_zmin) / 2);
-	Atom.NetToCartesian(m_a, m_b, m_c, alpha, beta, gamma);
-	
 	//Manage cleavages
 	CrystalCleavageList::iterator k;
 	for (k = Cleavages.begin(); k != Cleavages.end(); k++)
@@ -425,24 +422,69 @@ void CrystalDoc::Update()
 		ScalarProducts.clear() ;
 	}
 	
-	//Transform coordinates to Cartesians and find maximum distance from center
-	gdouble x, y, z, d;
-	Atom.GetCoords(&x, &y, &z);
-	m_MaxDist = 0;
-	for (i = Atoms.begin(); i != Atoms.end(); i++)
-	{
+	//Transform coordinates to Cartesians and find center of visible view
+	gdouble x, y, z, d,
+		xmin = G_MAXDOUBLE, ymin = G_MAXDOUBLE, zmin = G_MAXDOUBLE,
+		xmax = -G_MAXDOUBLE, ymax = -G_MAXDOUBLE, zmax = -G_MAXDOUBLE;
+	iend = Atoms.end ();
+	for (i = Atoms.begin(); i != iend; i++) {
 		(*i)->NetToCartesian(m_a, m_b, m_c, alpha, beta, gamma);
-		d =  (*i)->Distance(x, y, z, m_bFixedSize);
-		m_MaxDist = __max(m_MaxDist, d);
-		(*i)->Move(- x, - y, - z);
+		if ((*i)->IsCleaved ())
+			continue;
+		(*i)->GetCoords (&x, &y, &z);
+		if (x < xmin)
+			xmin = x;
+		if (y < ymin)
+			ymin = y;
+		if (z < zmin)
+			zmin = z;
+		if (x > xmax)
+			xmax = x;
+		if (y > ymax)
+			ymax = y;
+		if (z > zmax)
+			zmax = z;
+	}
+	jend = Lines.end ();
+	for (j = Lines.begin (); j != jend; j++) {
+		(*j)->NetToCartesian (m_a, m_b, m_c, alpha, beta, gamma);
+		if ((*j)->IsCleaved ())
+			continue;
+		x = (*j)->Xmin ();
+		y = (*j)->Ymin ();
+		z = (*j)->Zmin ();
+		if (x < xmin)
+			xmin = x;
+		if (y < ymin)
+			ymin = y;
+		if (z < zmin)
+			zmin = z;
+		x = (*j)->Xmax ();
+		y = (*j)->Ymax ();
+		z = (*j)->Zmax ();
+		if (x > xmax)
+			xmax = x;
+		if (y > ymax)
+			ymax = y;
+		if (z > zmax)
+			zmax = z;
 	}
 
-	for (j = Lines.begin(); j != Lines.end(); j++)
-	{
-		(*j)->NetToCartesian(m_a, m_b, m_c, alpha, beta, gamma);
-		d =  (*j)->Distance(x, y, z, m_bFixedSize);
-		m_MaxDist = __max(m_MaxDist, d);
-		(*j)->Move(- x, - y, - z);
+	//Searching the center of the crystal and find maximum distance from center
+	x = (xmin + xmax) / 2.;
+	y = (ymin + ymax) / 2.;
+	z = (zmin + zmax) / 2.;
+	m_MaxDist = 0;
+	for (i = Atoms.begin(); i != iend; i++) {
+		d =  (*i)->Distance (x, y, z, m_bFixedSize);
+		m_MaxDist = __max (m_MaxDist, d);
+		(*i)->Move (- x, - y, - z);
+	}
+
+	for (j = Lines.begin(); j != Lines.end(); j++) {
+		d =  (*j)->Distance (x, y, z, m_bFixedSize);
+		m_MaxDist = __max (m_MaxDist, d);
+		(*j)->Move (- x, - y, - z);
 	}
 }
 
@@ -490,11 +532,6 @@ void CrystalDoc::Duplicate(CrystalLine& Line)
 	}
 }
 
-void CrystalDoc::SetDirty()
-{
-	m_bDirty = true;
-}
-
 void CrystalDoc::Draw()
 {
 	CrystalAtomList::iterator i;
@@ -533,6 +570,7 @@ xmlDocPtr CrystalDoc::BuildXMLTree()
 	gchar buf[256];
 	xmlDocPtr xml;
 	xmlNodePtr node;
+	xmlNsPtr ns;
 	char *old_num_locale;
 
 	xml = xmlNewDoc((xmlChar*)"1.0");
@@ -541,7 +579,9 @@ xmlDocPtr CrystalDoc::BuildXMLTree()
 	old_num_locale = g_strdup(setlocale(LC_NUMERIC, NULL));
 	setlocale(LC_NUMERIC, "C");
 	xmlDocSetRootElement (xml,  xmlNewDocNode(xml, NULL, (xmlChar*)"crystal", NULL));
-	
+	ns = xmlNewNs (xml->children, (xmlChar*) "http://www.nongnu.org/gcrystal", (xmlChar*) "gcry");
+	xmlSetNs (xml->children, ns);
+
 	try
 	{
 		node = xmlNewDocNode(xml, NULL, (xmlChar*)"generator", (xmlChar*)GetProgramId());
@@ -569,6 +609,8 @@ xmlDocPtr CrystalDoc::BuildXMLTree()
 		if (node) xmlAddChild(xml->children, node); else throw (int) 0;
 		WritePosition(xml, node, "start", m_xmin, m_ymin, m_zmin);
 		WritePosition(xml, node, "end", m_xmax, m_ymax, m_zmax);
+		if (m_bFixedSize)
+			xmlNewProp (node, (xmlChar *) "fixed", (xmlChar *) "true");
 		
 		CrystalAtomList::iterator i;
 		for (i = AtomDef.begin(); i != AtomDef.end(); i++)
