@@ -29,7 +29,6 @@
 #include <glib.h>
 #include "chemistry.h"
 #include "crystalatom.h"
-#include "crystalbond.h"
 #include "crystalline.h"
 #include "crystalcleavage.h"
 #include "document.h"
@@ -256,6 +255,8 @@ List of the views of the document.
 	list <CrystalView *> m_Views;
 };
 
+extern gchar *LatticeName[];
+
 } //namespace gcu
 
 #endif //CRYSTAL_DOC_H
