@@ -4,7 +4,7 @@
  * Gnome Chemistry Utils
  * chemistry/document.cc
  *
- * Copyright (C) 2004 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2004-2007 Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -22,16 +22,27 @@
  * USA
  */
 
+#include "config.h"
 #include "document.h"
+#include "application.h"
+#include "dialog.h"
+#include <cstring>
 
 using namespace gcu;
 
-Document::Document (): Object (DocumentType)
+Document::Document (Application *App): Object (DocumentType),
+m_Dirty (false),
+m_Empty (true)
 {
+	m_App = App;
+	if (m_App)
+		m_App->AddDocument (this);
 }
 
 Document::~Document ()
 {
+	if (m_App)
+		m_App->RemoveDocument (this);
 }
 
 gchar* Document::GetNewId (gchar* id, bool Cache)
