@@ -2,7 +2,7 @@
  * Gnome Chemistry Utils
  * element.cc 
  *
- * Copyright (C) 2002-2006 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2002-2007 Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -27,16 +27,16 @@
 #include <libxml/parserInternals.h>
 #include <libxml/xmlmemory.h>
 #include <glib.h>
-#include <locale.h>
+#include <libintl.h>
+#include <glib/gi18n-lib.h>
+#include <goffice/utils/go-math.h>
+#include <clocale>
+#include <cmath>
+#include <cstring>
 #include <vector>
 #include <map>
 #include <set>
 #include <string>
-#include <libintl.h>
-#include <math.h>
-#include <string.h>
-#include <glib/gi18n-lib.h>
-#include <goffice/utils/go-math.h>
 
 static set<string>units;
 
@@ -769,9 +769,7 @@ void Element::LoadAllData ()
 	LoadRadii ();
 	LoadElectronicProps ();
 	LoadIsotopes ();
-#ifdef WITH_BODR
 	LoadBODR ();
-#endif
 }
 
 IsotopicPattern *Element::GetIsotopicPattern (unsigned natoms)
@@ -824,7 +822,6 @@ GcuDimensionalValue const *Element::GetElectronAffinity (unsigned rank)
 	return (rank <= m_ae.size ())? &m_ae[rank - 1]: NULL;
 }
 
-#ifdef WITH_BODR
 void Element::LoadBODR ()
 {
 	char *old_num_locale;
@@ -866,8 +863,7 @@ void Element::LoadBODR ()
 							if (unit) {
 								DimensionalValue *v = new DimensionalValue ();
 								if (!strcmp (unit, "units:atmass"))
-									// amu stands for "atomic mass units"
-									v->val.unit = _("amu");
+									v->val.unit = "u";
 								else if (!strcmp (unit, "units:ev"))
 									v->val.unit = "eV";
 								else if (!strcmp (unit, "units:ang"))
@@ -940,7 +936,6 @@ void Element::LoadBODR ()
 	setlocale (LC_NUMERIC, old_num_locale);
 	g_free (old_num_locale);
 }
-#endif
 
 int Element::GetIntegerProperty (char const *property_name)
 {
