@@ -4,9 +4,7 @@
  * Gnome Chemistry Utils
  * matrix2d.cc 
  *
- * Copyright (C) 2004
- *
- * Developed by Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2004 Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -24,8 +22,8 @@
  * USA
  */
 
-#include <math.h>
 #include "matrix2d.h"
+#include <cmath>
 
 using namespace gcu;
 
