@@ -4,7 +4,7 @@
  * Gnome Crystal
  * document.cc 
  *
- * Copyright (C) 2000-2006 Jean Brfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2000-2007 Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -25,12 +25,12 @@
 #include "config.h"
 #include "gcrystal.h"
 #include <unistd.h>
-#include <locale.h>
 #include <GL/gl.h>
 #include <GL/glu.h>
 #include "application.h"
 #include "document.h"
 #include "view.h"
+#include "window.h"
 #include "celldlg.h"
 #include "atomsdlg.h"
 #include "linesdlg.h"
@@ -39,9 +39,10 @@
 #include "globals.h"
 #include <gcu/filechooser.h>
 #include <glade/glade.h>
-#include <math.h>
 #include <libxml/parserInternals.h>
 #include <libxml/xmlmemory.h>
+//#include <libgnomevfs/gnome-vfs-file-info.h>
+#include <libgnomevfs/gnome-vfs-ops.h>
 #include <vector>
 #include <map>
 #ifdef HAVE_FSTREAM
@@ -54,6 +55,15 @@
 #else
 #	include <ostream.h>
 #endif
+#ifdef HAVE_SSTREAM
+#	include <sstream>
+#else
+#	include <sstream.h>
+#endif
+#include <glib/gi18n.h>
+#include <clocale>
+#include <cmath>
+#include <cstring>
 
 #define SAVE	1
 #define LOAD	2
@@ -64,38 +74,15 @@
 
 using namespace std;
 
-char *LatticeName[] = {"simple cubic",
-	"body-centered cubic",
-	"face-centered cubic",
-	"hexagonal",
-	"tetragonal",
-	"body-centered tetragonal",
-	"orthorhombic",
-	"base-centered orthorhombic",
-	"body-centered orthorhombic",
-	"face-centered orthorhombic",
-	"rhombohedral",
-	"monoclinic",
-	"base-centered monoclinic",
-	"triclinic"};
-
-gcDocument::gcDocument():CrystalDoc()
+gcDocument::gcDocument (gcApplication *pApp) :CrystalDoc (pApp)
 {
-	Init();
+	Init ();
 	m_filename = NULL;
 	m_title = NULL;
-	m_bEmpty = true;
-	m_bDirty = false;
 	m_bClosing = false;
-	m_ps = NULL;
+	m_ReadOnly = false;
 }
 
-/*gcDocument::gcDocument(bool create_view)
-{
-	Init();
-	if (create_view) pView = m_pView = new gcView(this);
-}*/
-
 gcDocument::~gcDocument()
 {
 	if (m_filename != NULL) g_free(m_filename);
@@ -114,19 +101,19 @@ void gcDocument::Define (unsigned nPage)
 {
 	switch(nPage) {
 	case 0:
-		new gcCellDlg (((gcView*) m_Views.front ())->GetApp (), this);
+		new gcCellDlg (dynamic_cast <gcApplication *> (m_App), this);
 		break;
 	case 1:
-		new gcAtomsDlg (((gcView*) m_Views.front ())->GetApp (), this);
+		new gcAtomsDlg (dynamic_cast <gcApplication *> (m_App), this);
 		break;
 	case 2:
-		new gcLinesDlg (((gcView*) m_Views.front ())->GetApp (), this);
+		new gcLinesDlg (dynamic_cast <gcApplication *> (m_App), this);
 		break;
 	case 3:
-		new gcSizeDlg (((gcView*) m_Views.front ())->GetApp (), this);
+		new gcSizeDlg (dynamic_cast <gcApplication *> (m_App), this);
 		break;
 	case 4:
-		new gcCleavagesDlg (((gcView*) m_Views.front ())->GetApp (), this);
+		new gcCleavagesDlg (dynamic_cast <gcApplication *> (m_App), this);
 		break;
 	}
 }
@@ -185,37 +172,39 @@ void gcDocument::SetCell(gcLattices lattice, gdouble a, gdouble b, gdouble c, gd
 	m_gamma = gamma;
 }
 
-void gcDocument::SetFileName(const gchar* filename)
+void gcDocument::SetFileName (const string &filename)
 {
-	if (m_filename) g_free(m_filename);
-	m_filename = g_strdup(filename);
-	int i = strlen(filename) - 1;
-	while ((m_filename[i] != '/') && (i >= 0)) i--;
-	if (i >=0) 
-	{
-		m_filename[i] = 0;
-		chdir(m_filename);
+	GnomeVFSFileInfo *info = gnome_vfs_file_info_new ();
+	gnome_vfs_get_file_info (filename.c_str (), info, GNOME_VFS_FILE_INFO_DEFAULT);
+	m_ReadOnly = !(info->permissions & (GNOME_VFS_PERM_USER_WRITE | GNOME_VFS_PERM_GROUP_WRITE));
+	gnome_vfs_file_info_unref (info);
+	if (m_filename)
+		g_free (m_filename);
+	m_filename = g_strdup (filename.c_str ());
+	char *dirname = g_path_get_dirname (m_filename);
+	m_App->SetCurDir (dirname);
+	g_free (dirname);
+	int i = filename.length () - 1;
+	while ((m_filename[i] != '/') && (i >= 0))
+		i--;
+	if (i >=0) {
+		m_filename [i] = 0;
+		chdir (m_filename);
 		m_filename[i] = '/';
 	}
 	i++;
-	int j = strlen(filename) - 1;
-	while ((i < j) && (m_filename[j] != '.')) j--;
-	gchar* title = (strcmp(m_filename + j, ".gcrystal"))? g_strdup(m_filename + i):g_strndup(m_filename + i, j - i);
-	SetTitle(title);
-	g_free(title);
+	int j = filename.length () - 1;
+	while ((i < j) && (m_filename[j] != '.'))
+		j--;
+	gchar* title = (strcmp (m_filename + j, ".gcrystal"))? g_strdup (m_filename + i):g_strndup (m_filename + i, j - i);
+	SetTitle (title);
+	g_free (title);
 }
 
 void gcDocument::SetTitle(const gchar* title)
 {
 	if (m_title) g_free(m_title);
 	m_title = g_strdup(title);
-	list<CrystalView*>::iterator view;
-	GtkLabel *label;
-	for (view = m_Views.begin(); view != m_Views.end(); view++)
-	{
-		label = ((gcView*)(*view))->GetLabel();
-		if (label) gtk_label_set_text(label, title);
-	}
 }
 
 void gcDocument::Save()
@@ -226,12 +215,13 @@ void gcDocument::Save()
 
 	try {
 		xml = BuildXMLTree();
-	
+
 		if (xmlSaveFile (m_filename, xml) < 0)
 			Error (SAVE);
 			
 		xmlFreeDoc (xml);
-		m_bDirty = false;
+		SetDirty (false);
+		m_ReadOnly = false;	// if saving succeded, the file is not read only...
 	}
 	catch (int num) {
 		xmlFreeDoc (xml);
@@ -263,7 +253,7 @@ void gcDocument::Error (int num)
 	g_free (mess);
 }
 
-bool gcDocument::Load (const gchar* filename)
+bool gcDocument::Load (const string &filename)
 {
 	xmlDocPtr xml = NULL;
 	gchar *oldfilename, *oldtitle;
@@ -274,7 +264,7 @@ bool gcDocument::Load (const gchar* filename)
 	try {
 		if (SetFileName (filename), !m_filename || !m_title)
 			throw (int) 0;
-		if (!(xml = xmlParseFile (filename)))
+		if (!(xml = xmlParseFile (filename.c_str ())))
 			throw (int) 1;
 		if (xml->children == NULL)
 			throw (int) 2;
@@ -405,20 +395,13 @@ void gcDocument::ParseXMLTree(xmlNode* xml)
 				if (pCleavage->LoadOld(node)) Cleavages.push_back((CrystalCleavage*)pCleavage);
 				else delete pCleavage;
 			}
-			else if (!strcmp((gchar*)node->name, "view"))
-			{
+			else if (!strcmp( (gchar*) node->name, "view")) {
 				if (bViewLoaded) {
-					gcView* pView = new gcView(this);
+					gcWindow *pWindow = new gcWindow (dynamic_cast <gcApplication *> (m_App), this);
+					gcView *pView = pWindow->GetView ();
 					pView->LoadOld(node);
-					m_Views.push_back(pView);
-					if (!RequestApp(pView))
-					{
-						delete pView;
-					}
-				}
-				else
-				{
-					m_Views.front()->Load(node); //the first view is created with the document
+				} else {
+					m_Views.front ()->Load (node); //the first view is created with the document
 					bViewLoaded = true;
 				}
 			}
@@ -439,124 +422,130 @@ void gcDocument::OnNewDocument()
 typedef struct {int n; std::list<CrystalAtom*> l;} sAtom;
 typedef struct {int n; std::list<CrystalLine*> l;} sLine;
 
-void gcDocument::OnExportVRML(const gchar* FileName, gcView* pView)
+void gcDocument::OnExportVRML (const string &FileName)
 {
 	char *old_num_locale, tmp[128];
 	double x0, x1, x2, x3, x4, x5;
 	int n = 0;
-	ofstream file(FileName);
-	std::map<std::string, sAtom /*list<gcAtom*>*/ >AtomsMap;
-	std::map<std::string, sLine>LinesMap;
-	if (!file)
-	{
-		gchar* mess = g_strdup_printf(_("Can't create file %s"), FileName);
-		GtkWidget* message = gtk_message_dialog_new(NULL, (GtkDialogFlags) 0, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, mess);
-		gtk_widget_show(message);
-		g_free(mess);
-	}
-	old_num_locale = g_strdup(setlocale(LC_NUMERIC, NULL));
-	setlocale(LC_NUMERIC, "C");
-	file << "#VRML V2.0 utf8" << endl;
-	
-	//Create prototypes for atoms
-	CrystalAtomList::iterator i;
-	for (i = Atoms.begin(); i != Atoms.end(); i++)
-	{
-		(*i)->GetColor(&x0, &x1, &x2, &x3);
-		snprintf(tmp, sizeof(tmp), "%g %g %g %g %g", (*i)->GetSize(), x0, x1, x2, x3);
-		if (AtomsMap[tmp].l.empty())
-		{
-			AtomsMap[tmp].n = n;
-			file << "PROTO Atom" << n++ << " [] {Shape {" << endl << "\tgeometry Sphere {radius " << (*i)->GetSize() / 100 << "}" << endl;
-			file << "\tappearance Appearance {" << endl << "\t\tmaterial Material {" << endl << "\t\t\tdiffuseColor " << x0 << " " << x1 << " " << x2 << endl;
-			if (x3 < 1) file << "\t\t\ttransparency " << (1 - x3) << endl;
-			file << "\t\t\tspecularColor 1 1 1" << endl << "\t\t\tshininess 0.9" << endl << "\t\t}" << endl << "\t}\r\n}}" << endl;
-		}
-		AtomsMap[tmp].l.push_back(*i);
-	}
+	try {
+		ostringstream file;
+		GnomeVFSHandle *handle = NULL;
+		GnomeVFSFileSize fs;
+		GnomeVFSResult res;
+		std::map<std::string, sAtom>AtomsMap;
+		std::map<std::string, sLine>LinesMap;
+		if ((res = gnome_vfs_create (&handle, FileName.c_str (), GNOME_VFS_OPEN_WRITE, true, 0644)) != GNOME_VFS_OK)
+			throw (int) res;
+		old_num_locale = g_strdup(setlocale(LC_NUMERIC, NULL));
+		setlocale(LC_NUMERIC, "C");
 
-	//Create prototypes for bonds
-	CrystalLineList::iterator j;
-	n = 0;
-	for (j = Lines.begin(); j != Lines.end(); j++)
-	{
-		(*j)->GetColor(&x0, &x1, &x2, &x3);
-		snprintf(tmp, sizeof(tmp), "%g %g %g %g %g %g", (*j)->Long(), (*j)->GetRadius(), x0, x1, x2, x3);
-		if (LinesMap[tmp].l.empty())
+		file << "#VRML V2.0 utf8" << endl;
+		
+		//Create prototypes for atoms
+		CrystalAtomList::iterator i;
+		for (i = Atoms.begin(); i != Atoms.end(); i++)
 		{
-			LinesMap[tmp].n = n;
-			file << "PROTO Bond" << n++ << " [] {Shape {" << endl << "\tgeometry Cylinder {radius " << (*j)->GetRadius() / 100 << "\theight " << (*j)->Long() / 100 << "}" << endl;
-			file << "\tappearance Appearance {" << endl << "\t\tmaterial Material {" << endl << "\t\t\tdiffuseColor " << x0 << " " << x1 << " " << x2 << endl;
-			if (x3 < 1) file << "\t\t\ttransparency " << (1 - x3) << endl;
-			file << "\t\t\tspecularColor 1 1 1" << endl << "\t\t\tshininess 0.9" << endl << "\t\t}" << endl << "\t}\r\n}}" << endl;
-		}
-		LinesMap[tmp].l.push_back(*j);
-	}
-	
-	//world begin
-	pView->GetBackgroundColor(&x0, &x1, &x2, &x3);
-	file << "Background{skyColor " << x0 << " " << x1 << " " << x2 << "}" << endl;
-	file << "Viewpoint {fieldOfView " << pView->GetFoV()/90*1.570796326794897 <<"\tposition 0 0 " << pView->GetPos() / 100 << "}" << endl;
-	pView->GetRotation(&x0, &x1, &x2);
-	Matrix m(x0/90*1.570796326794897, x1/90*1.570796326794897, x2/90*1.570796326794897, euler);
-	file << "Transform {" << endl << "\tchildren [" << endl;
-
-	std::map<std::string, sAtom>::iterator k;
-	for (k = AtomsMap.begin(); k != AtomsMap.end(); k++)
-	{
-		for (i = (*k).second.l.begin(); i != (*k).second.l.end(); i++)
-		{
-			if (!(*i)->IsCleaved())
+			(*i)->GetColor(&x0, &x1, &x2, &x3);
+			snprintf(tmp, sizeof(tmp), "%g %g %g %g %g", (*i)->GetSize(), x0, x1, x2, x3);
+			if (AtomsMap[tmp].l.empty())
 			{
-				x0 = (*i)->x();
-				x1 = (*i)->y();
-				x2 = (*i)->z();
-				m.Transform(x0, x1, x2);
-				file << "\t\tTransform {translation " << x1 / 100 << " " << x2 / 100 << " " << x0 / 100\
-					<<  " children [Atom" << (*k).second.n << " {}]}" << endl;
+				AtomsMap[tmp].n = n;
+				file << "PROTO Atom" << n++ << " [] {Shape {" << endl << "\tgeometry Sphere {radius " << (*i)->GetSize() / 100 << "}" << endl;
+				file << "\tappearance Appearance {" << endl << "\t\tmaterial Material {" << endl << "\t\t\tdiffuseColor " << x0 << " " << x1 << " " << x2 << endl;
+				if (x3 < 1) file << "\t\t\ttransparency " << (1 - x3) << endl;
+				file << "\t\t\tspecularColor 1 1 1" << endl << "\t\t\tshininess 0.9" << endl << "\t\t}" << endl << "\t}\r\n}}" << endl;
 			}
+			AtomsMap[tmp].l.push_back(*i);
 		}
-		(*k).second.l.clear();
-	}
-	AtomsMap.clear();
 	
-	std::map<std::string, sLine>::iterator l;
-	n = 0;
-	for (l = LinesMap.begin(); l != LinesMap.end(); l++)
-	{
-		for (j = (*l).second.l.begin(); j != (*l).second.l.end(); j++)
+		//Create prototypes for bonds
+		CrystalLineList::iterator j;
+		n = 0;
+		for (j = Lines.begin(); j != Lines.end(); j++)
 		{
-			if (!(*j)->IsCleaved())
+			(*j)->GetColor(&x0, &x1, &x2, &x3);
+			snprintf(tmp, sizeof(tmp), "%g %g %g %g %g %g", (*j)->Long(), (*j)->GetRadius(), x0, x1, x2, x3);
+			if (LinesMap[tmp].l.empty())
 			{
-				x0 = (*j)->X1();
-				x1 = (*j)->Y1();
-				x2 = (*j)->Z1();
-				m.Transform(x0, x1, x2);
-				x3 = (*j)->X2();
-				x4 = (*j)->Y2();
-				x5 = (*j)->Z2();
-				m.Transform(x3, x4, x5);
-				CrystalLine line(gcu::unique, x0, x1, x2, x3, x4, x5, 0.0, 0.0, 0.0, 0.0, 0.0);
-				line.GetRotation(x0, x1, x2, x3);
-				file << "\t\tTransform {" << endl << "\t\t\trotation " << x1 << " " << x2 << " " << x0 << " " << x3 << endl;
-				x0 = (line.X1() + line.X2()) / 200;
-				x1 = (line.Y1() + line.Y2()) / 200;
-				x2 = (line.Z1() + line.Z2()) / 200;
-				file << "\t\t\ttranslation " << x1 << " " << x2  << " " << x0 <<  endl\
-						<< "\t\t\tchildren [Bond" << (*l).second.n << " {}]}" << endl;
+				LinesMap[tmp].n = n;
+				file << "PROTO Bond" << n++ << " [] {Shape {" << endl << "\tgeometry Cylinder {radius " << (*j)->GetRadius() / 100 << "\theight " << (*j)->Long() / 100 << "}" << endl;
+				file << "\tappearance Appearance {" << endl << "\t\tmaterial Material {" << endl << "\t\t\tdiffuseColor " << x0 << " " << x1 << " " << x2 << endl;
+				if (x3 < 1) file << "\t\t\ttransparency " << (1 - x3) << endl;
+				file << "\t\t\tspecularColor 1 1 1" << endl << "\t\t\tshininess 0.9" << endl << "\t\t}" << endl << "\t}\r\n}}" << endl;
 			}
+			LinesMap[tmp].l.push_back(*j);
 		}
-		n++;
-		(*l).second.l.clear();
-	}
-	LinesMap.clear();
+		
+		//world begin
+		m_pActiveView->GetBackgroundColor(&x0, &x1, &x2, &x3);
+		file << "Background{skyColor " << x0 << " " << x1 << " " << x2 << "}" << endl;
+		file << "Viewpoint {fieldOfView " << m_pActiveView->GetFoV()/90*1.570796326794897 <<"\tposition 0 0 " << m_pActiveView->GetPos() / 100 << "}" << endl;
+		m_pActiveView->GetRotation(&x0, &x1, &x2);
+		Matrix m(x0/90*1.570796326794897, x1/90*1.570796326794897, x2/90*1.570796326794897, euler);
+		file << "Transform {" << endl << "\tchildren [" << endl;
 	
-	//end of the world
-	file << "\t]" << endl << "}" << endl;
+		std::map<std::string, sAtom>::iterator k;
+		for (k = AtomsMap.begin(); k != AtomsMap.end(); k++)
+		{
+			for (i = (*k).second.l.begin(); i != (*k).second.l.end(); i++)
+			{
+				if (!(*i)->IsCleaved())
+				{
+					x0 = (*i)->x();
+					x1 = (*i)->y();
+					x2 = (*i)->z();
+					m.Transform(x0, x1, x2);
+					file << "\t\tTransform {translation " << x1 / 100 << " " << x2 / 100 << " " << x0 / 100\
+						<<  " children [Atom" << (*k).second.n << " {}]}" << endl;
+				}
+			}
+			(*k).second.l.clear();
+		}
+		AtomsMap.clear();
+		
+		std::map<std::string, sLine>::iterator l;
+		n = 0;
+		for (l = LinesMap.begin(); l != LinesMap.end(); l++)
+		{
+			for (j = (*l).second.l.begin(); j != (*l).second.l.end(); j++)
+			{
+				if (!(*j)->IsCleaved())
+				{
+					x0 = (*j)->X1();
+					x1 = (*j)->Y1();
+					x2 = (*j)->Z1();
+					m.Transform(x0, x1, x2);
+					x3 = (*j)->X2();
+					x4 = (*j)->Y2();
+					x5 = (*j)->Z2();
+					m.Transform(x3, x4, x5);
+					CrystalLine line(gcu::unique, x0, x1, x2, x3, x4, x5, 0.0, 0.0, 0.0, 0.0, 0.0);
+					line.GetRotation(x0, x1, x2, x3);
+					file << "\t\tTransform {" << endl << "\t\t\trotation " << x1 << " " << x2 << " " << x0 << " " << x3 << endl;
+					x0 = (line.X1() + line.X2()) / 200;
+					x1 = (line.Y1() + line.Y2()) / 200;
+					x2 = (line.Z1() + line.Z2()) / 200;
+					file << "\t\t\ttranslation " << x1 << " " << x2  << " " << x0 <<  endl\
+							<< "\t\t\tchildren [Bond" << (*l).second.n << " {}]}" << endl;
+				}
+			}
+			n++;
+			(*l).second.l.clear();
+		}
+		LinesMap.clear();
+		
+		//end of the world
+		file << "\t]" << endl << "}" << endl;
 
-	file.close();
-	setlocale(LC_NUMERIC, old_num_locale);
-	g_free(old_num_locale);
+		setlocale(LC_NUMERIC, old_num_locale);
+		g_free(old_num_locale);
+		if ((res = gnome_vfs_write (handle, file.str ().c_str (), (GnomeVFSFileSize) file.str ().size (), &fs)) != GNOME_VFS_OK)
+			throw (int) res;
+		gnome_vfs_close (handle);
+	}
+	catch (int n) {
+		fprintf (stderr, "gnome-vfs error #%d\n",n);
+	}
 }
 
 gcView *gcDocument::GetNewView()
@@ -564,39 +553,45 @@ gcView *gcDocument::GetNewView()
 	return NULL;
 }
 
-void gcDocument::SetDirty()
-{
-	m_bDirty = true;
-	if (m_ps) bonobo_persist_set_dirty(m_ps, m_bDirty);
-}
-
 void gcDocument::AddView(gcView* pView)
 {
-	m_Views.push_back(pView);
-	if (!m_bEmpty) m_bDirty = true;
+	m_Views.push_back (pView);
+	RenameViews ();
+	if (!GetEmpty ())
+		SetDirty (true);
 }
 
-bool gcDocument::RemoveView(gcView* pView)
+bool gcDocument::RemoveView (gcView* pView)
 {
-	if (pView->IsLocked()) return false;
-	if (m_Views.size() > 1)
-	{
-		m_Views.remove(pView);
-		if (!m_bClosing && !m_bEmpty) m_bDirty = true;
+	if (m_Views.size () > 1) {
+		m_Views.remove (pView);
+		RenameViews ();
+		if (!m_bClosing && !GetEmpty ())
+			SetDirty (true);
 		return true;
 	}
-	if (IsDirty())
-	{
-		if (!VerifySaved()) return false;
+	if (GetDirty ()) {
+		if (!VerifySaved ())
+			return false;
 	}
-	RemoveDocument(this);
+	dynamic_cast <gcApplication *> (m_App)->RemoveDocument (this);
+	delete this;
 	return true;
 }
 
+void gcDocument::RemoveAllViews ()
+{
+	while (m_Views.size () > 1)
+		dynamic_cast<gcView*> (m_Views.front ())->GetWindow ()->Destroy ();
+	// The last one is deleted separately since this will be destroyed !
+	dynamic_cast<gcView*> (m_Views.front ())->GetWindow ()->Destroy ();
+}
+
 bool gcDocument::VerifySaved()
 {
 	m_bClosing = true;
-	if (!m_bDirty) return true;
+	if (!GetDirty ())
+		return true;
 	gchar* str = g_strdup_printf(_("\"%s\" has been modified.  Do you wish to save it?"), m_title);
 	GtkWidget* mbox;
 	int res;
@@ -608,34 +603,22 @@ bool gcDocument::VerifySaved()
 		gtk_widget_destroy(mbox);
 		if (res == GTK_RESPONSE_YES) {
 			if (m_filename == NULL) {
-				list<char const*> l;
+				list<string> l;
 				l.push_front ("application/x-gcrystal");
-				FileChooser (((gcView*) m_Views.front ())->GetApp (), true, l, this);
-				while (((gcView*) (m_Views.front ()))->IsLocked ())
-					if (gtk_events_pending ())
-						gtk_main_iteration ();
+				FileChooser (m_App, true, l, this);
 			}
 			if (m_filename)
 				Save ();
 		}
 	}
 	while ((res == GTK_RESPONSE_YES) && (m_filename == NULL));
-	if (res == GTK_RESPONSE_NO) m_bDirty = false;
+	if (res == GTK_RESPONSE_NO)
+		SetDirty (false);
 	else if (res == GTK_RESPONSE_CANCEL) m_bClosing = false;
 	g_free(str);
 	return (res != GTK_RESPONSE_CANCEL);
 }
 
-void gcDocument::NotifyDialog (Dialog* dialog)
-{
-	m_Dialogs.push_front(dialog);
-}
-
-void gcDocument::RemoveDialog (Dialog* dialog)
-{
-	m_Dialogs.remove(dialog);
-}
-
 CrystalView* gcDocument::CreateNewView()
 {
 	return new gcView(this);
@@ -660,3 +643,38 @@ const char* gcDocument::GetProgramId()
 {
 	return "Gnome Crystal "VERSION;
 }
+
+void gcDocument::SaveAsImage (const string &filename, char const *type, map<string, string>& options)
+{
+	m_pActiveView->SaveAsImage (filename, type, options, GetApp ()->GetImageWidth (), GetApp ()->GetImageHeight ());
+}
+
+bool gcDocument::LoadNewView (xmlNodePtr node)
+{
+	gcWindow *pWindow = new gcWindow (dynamic_cast <gcApplication *> (m_App), this);
+	gcView *pView = pWindow->GetView ();
+	bool result = pView->Load (node);
+	if (!result)
+		delete pWindow;
+	return result;
+}
+
+void gcDocument::RenameViews ()
+{
+	list <CrystalView *>::iterator i, iend = m_Views.end ();
+	int n = 1, max = m_Views.size ();
+	for (i = m_Views.begin (); i != iend; i++) {
+		gcWindow *window = dynamic_cast <gcView*> (*i)->GetWindow ();
+		GtkWindow *w = window->GetWindow ();
+		if (!w)
+			continue;
+		if (max > 1) {
+			char *t = g_strdup_printf ("%s (%i)", m_title, n++);
+			gtk_window_set_title (w, t);
+			g_free (t);
+		} else
+			gtk_window_set_title (w, m_title);
+		window->ActivateActionWidget ("ui/MainMenu/FileMenu/Save", !m_ReadOnly);
+		window->ActivateActionWidget ("ui/MainToolbar/Save", !m_ReadOnly);
+	}
+}
