@@ -4,7 +4,7 @@
  * Gnome Crystal
  * linesdlg.cc 
  *
- * Copyright (C) 2002-2006 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2002-2007 Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -90,10 +90,14 @@ static void on_medians_toggled (GtkToggleButton* btn, gcLinesDlg *pBox)
 	pBox->OnToggledSpecial (medians);
 }
 
-gcLinesDlg::gcLinesDlg (gcApplication *App, gcDocument* pDoc): Dialog (App, GLADEDIR"/lines.glade", "lines")
+gcLinesDlg::gcLinesDlg (gcApplication *App, gcDocument* pDoc): Dialog (App, GLADEDIR"/lines.glade", "lines", pDoc)
 {
+	if (!xml) {
+		m_Lines = NULL;
+		delete this;
+		return;
+	}
 	m_pDoc = pDoc;
-	pDoc->NotifyDialog (this);
 	GtkWidget* button = glade_xml_get_widget (xml, "add");
 	g_signal_connect (G_OBJECT (button), "clicked", GTK_SIGNAL_FUNC (on_add), this);
 	DeleteBtn = glade_xml_get_widget (xml, "delete");
@@ -112,7 +116,7 @@ gcLinesDlg::gcLinesDlg (gcApplication *App, gcDocument* pDoc): Dialog (App, GLAD
 	g_signal_connect (G_OBJECT (renderer), "edited", G_CALLBACK (on_edited), this);
 	g_object_set (G_OBJECT (renderer), "editable", true, NULL);
 	g_object_set_data (G_OBJECT (renderer), "column", (gint *) COLUMN_X1);
-	column = gtk_tree_view_column_new_with_attributes (_("x1"), renderer, "text", COLUMN_X1, NULL);
+	column = gtk_tree_view_column_new_with_attributes ("x1", renderer, "text", COLUMN_X1, NULL);
 	/* set this column to a fixed sizing (of 50 pixels) */
 	gtk_tree_view_column_set_sizing (GTK_TREE_VIEW_COLUMN (column), GTK_TREE_VIEW_COLUMN_FIXED);
 	gtk_tree_view_column_set_fixed_width (GTK_TREE_VIEW_COLUMN (column), 50);
@@ -122,7 +126,7 @@ gcLinesDlg::gcLinesDlg (gcApplication *App, gcDocument* pDoc): Dialog (App, GLAD
 	g_signal_connect (G_OBJECT (renderer), "edited", G_CALLBACK (on_edited), this);
 	g_object_set (G_OBJECT (renderer), "editable", true, NULL);
 	g_object_set_data (G_OBJECT (renderer), "column", (gint *) COLUMN_Y1);
-	column = gtk_tree_view_column_new_with_attributes (_("y1"), renderer, "text", COLUMN_Y1, NULL);
+	column = gtk_tree_view_column_new_with_attributes ("y1", renderer, "text", COLUMN_Y1, NULL);
 	/* set this column to a fixed sizing (of 50 pixels) */
 	gtk_tree_view_column_set_sizing (GTK_TREE_VIEW_COLUMN (column), GTK_TREE_VIEW_COLUMN_FIXED);
 	gtk_tree_view_column_set_fixed_width (GTK_TREE_VIEW_COLUMN (column), 50);
@@ -132,7 +136,7 @@ gcLinesDlg::gcLinesDlg (gcApplication *App, gcDocument* pDoc): Dialog (App, GLAD
 	g_signal_connect (G_OBJECT (renderer), "edited", G_CALLBACK (on_edited), this);
 	g_object_set (G_OBJECT (renderer), "editable", true, NULL);
 	g_object_set_data (G_OBJECT (renderer), "column", (gint *) COLUMN_Z1);
-	column = gtk_tree_view_column_new_with_attributes (_("z1"), renderer, "text", COLUMN_Z1, NULL);
+	column = gtk_tree_view_column_new_with_attributes ("z1", renderer, "text", COLUMN_Z1, NULL);
 	/* set this column to a fixed sizing (of 50 pixels) */
 	gtk_tree_view_column_set_sizing (GTK_TREE_VIEW_COLUMN (column), GTK_TREE_VIEW_COLUMN_FIXED);
 	gtk_tree_view_column_set_fixed_width (GTK_TREE_VIEW_COLUMN (column), 50);
@@ -142,7 +146,7 @@ gcLinesDlg::gcLinesDlg (gcApplication *App, gcDocument* pDoc): Dialog (App, GLAD
 	g_signal_connect (G_OBJECT (renderer), "edited", G_CALLBACK (on_edited), this);
 	g_object_set(G_OBJECT(renderer), "editable", true, NULL);
 	g_object_set_data(G_OBJECT (renderer), "column", (gint *)COLUMN_X2);
-	column = gtk_tree_view_column_new_with_attributes (_("x2"), renderer, "text", COLUMN_X2, NULL);
+	column = gtk_tree_view_column_new_with_attributes ("x2", renderer, "text", COLUMN_X2, NULL);
 	/* set this column to a fixed sizing (of 50 pixels) */
 	gtk_tree_view_column_set_sizing (GTK_TREE_VIEW_COLUMN (column), GTK_TREE_VIEW_COLUMN_FIXED);
 	gtk_tree_view_column_set_fixed_width (GTK_TREE_VIEW_COLUMN (column), 50);
@@ -152,7 +156,7 @@ gcLinesDlg::gcLinesDlg (gcApplication *App, gcDocument* pDoc): Dialog (App, GLAD
 	g_signal_connect (G_OBJECT (renderer), "edited", G_CALLBACK (on_edited), this);
 	g_object_set (G_OBJECT (renderer), "editable", true, NULL);
 	g_object_set_data (G_OBJECT (renderer), "column", (gint *) COLUMN_Y2);
-	column = gtk_tree_view_column_new_with_attributes (_("y2"), renderer, "text", COLUMN_Y2, NULL);
+	column = gtk_tree_view_column_new_with_attributes ("y2", renderer, "text", COLUMN_Y2, NULL);
 	/* set this column to a fixed sizing (of 50 pixels) */
 	gtk_tree_view_column_set_sizing (GTK_TREE_VIEW_COLUMN (column), GTK_TREE_VIEW_COLUMN_FIXED);
 	gtk_tree_view_column_set_fixed_width (GTK_TREE_VIEW_COLUMN (column), 50);
@@ -162,7 +166,7 @@ gcLinesDlg::gcLinesDlg (gcApplication *App, gcDocument* pDoc): Dialog (App, GLAD
 	g_signal_connect (G_OBJECT (renderer), "edited", G_CALLBACK (on_edited), this);
 	g_object_set (G_OBJECT (renderer), "editable", true, NULL);
 	g_object_set_data (G_OBJECT (renderer), "column", (gint *) COLUMN_Z2);
-	column = gtk_tree_view_column_new_with_attributes (_("z2"), renderer, "text", COLUMN_Z2, NULL);
+	column = gtk_tree_view_column_new_with_attributes ("z2", renderer, "text", COLUMN_Z2, NULL);
 	/* set this column to a fixed sizing (of 50 pixels) */
 	gtk_tree_view_column_set_sizing (GTK_TREE_VIEW_COLUMN (column), GTK_TREE_VIEW_COLUMN_FIXED);
 	gtk_tree_view_column_set_fixed_width (GTK_TREE_VIEW_COLUMN (column), 50);
@@ -273,7 +277,8 @@ gcLinesDlg::gcLinesDlg (gcApplication *App, gcDocument* pDoc): Dialog (App, GLAD
 
 gcLinesDlg::~gcLinesDlg ()
 {
-	m_pDoc->RemoveDialog (this);
+	if (m_Lines)
+		g_array_free (m_Lines, false);
 }
 
 bool gcLinesDlg::Apply ()
@@ -342,7 +347,7 @@ bool gcLinesDlg::Apply ()
 		Lines->push_back (new CrystalLine((s->duplicated)? normal: gcu::unique, s->x1, s->y1, s->z1, s->x2, s->y2, s->z2, s->r, (float)s->Red, (float)s->Green, (float)s->Blue, (float)s->Alpha));
 	}
 	m_pDoc->Update ();
-	m_pDoc->SetDirty ();
+	m_pDoc->SetDirty (true);
 	return true;
 }
 
@@ -429,6 +434,7 @@ void gcLinesDlg::LineSelect (GtkTreeSelection *Selection)
 		tmp = g_strdup_printf ("%g", g_array_index (m_Lines, struct LineStruct, m_LineSelected).r);
 		gtk_entry_set_text (LineR, tmp);
 		g_free (tmp);
+		gtk_tree_path_free (path);
 	} else {
 		gtk_widget_set_sensitive (DeleteBtn, false);
 		if (!m_Lines->len) gtk_widget_set_sensitive (DeleteAllBtn, false);
@@ -466,6 +472,7 @@ void gcLinesDlg::OnEdited (GtkCellRendererText *cell, const gchar *path_string,
 		g_array_index (m_Lines, struct LineStruct, m_LineSelected).z2 = x;
 		break;
 	}
+	gtk_tree_path_free (path);
 }
 
 void gcLinesDlg::OnToggled (GtkCellRendererToggle *cell, const gchar *path_string)
@@ -478,7 +485,8 @@ void gcLinesDlg::OnToggled (GtkCellRendererToggle *cell, const gchar *path_strin
 	gtk_tree_model_get (GTK_TREE_MODEL (LineList), &iter, COLUMN_SINGLE, &single, -1);
 	single ^= 1;
 	gtk_list_store_set (LineList, &iter, COLUMN_SINGLE, single, -1);
-	g_array_index (m_Lines, struct LineStruct, m_LineSelected).duplicated = !single;
+	g_array_index (m_Lines, struct LineStruct, gtk_tree_path_get_indices (path)[0]).duplicated = !single;
+	gtk_tree_path_free (path);
 }
 
 void gcLinesDlg::OnToggledSpecial (int Type)
