@@ -4,7 +4,7 @@
  * Gnome Crystal
  * prefs.cc 
  *
- * Copyright (C) 2001-2006 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2001-2007 Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -34,11 +34,16 @@ static void on_print_resolution (GtkWidget *widget, gcPrefsDlg * dialog)
 	dialog->UpdatePrinting ();
 }
 
-gcPrefsDlg::gcPrefsDlg (gcApplication *App): Dialog (App, GLADEDIR"/prefs.glade", "prefs")
+gcPrefsDlg::gcPrefsDlg (gcApplication *App): Dialog (App, GLADEDIR"/prefs.glade", "prefs", App)
 {
+	if (!xml) {
+		delete this;
+		return;
+	}
 	PrintResMenu = (GtkComboBox *) glade_xml_get_widget (xml, "printres");
 	PrintResBtn = (GtkSpinButton *) glade_xml_get_widget (xml, "printresbtn");
 	int PrintIndex;
+	bool active = false;
 	switch (PrintResolution) {
 	case 300:
 		PrintIndex = 0;
@@ -66,9 +71,11 @@ gcPrefsDlg::gcPrefsDlg (gcApplication *App): Dialog (App, GLADEDIR"/prefs.glade"
 		break;
 	default:
 		PrintIndex = 8;
-		gtk_spin_button_set_value (PrintResBtn, PrintResolution);
+		active = true;
 		break;
 	}
+	gtk_spin_button_set_value (PrintResBtn, PrintResolution);
+	gtk_widget_set_sensitive (GTK_WIDGET (PrintResBtn), active);
 	gtk_combo_box_set_active (PrintResMenu, PrintIndex);
 	g_signal_connect (PrintResMenu, "changed", G_CALLBACK (on_print_resolution), this);
 	FoVBtn = GTK_SPIN_BUTTON (glade_xml_get_widget (xml, "fov"));
@@ -128,7 +135,7 @@ bool gcPrefsDlg::Apply()
 
 void gcPrefsDlg::UpdatePrinting ()
 {
-	int PrintRes;
+	int PrintRes = PrintResolution ;
 	int PrintIndex = gtk_combo_box_get_active (PrintResMenu);
 	switch (PrintIndex)
 	{
